@@ -3,7 +3,7 @@ use std::fmt;
 /// Http header
 ///
 /// Has a name and a value.
-#[derive(Hash, PartialEq, Eq)]
+#[derive(Hash, Clone, PartialEq, Eq)]
 pub struct Header {
     /// Name of the Header
     pub name: String,
@@ -33,24 +33,6 @@ impl Header {
         }
     }
 
-    /// Convert a header ref to a header
-    /// ## Example
-    /// ```rust
-    /// // Import Modules
-    /// use afire::Header;
-    ///
-    /// let header1 = &Header::new("Content-Type", "text/html");
-    /// let header2 = Header::new("Content-Type", "text/html");
-    ///
-    /// assert!(header1.copy() == header2);
-    /// ```
-    pub fn copy(&self) -> Header {
-        Header {
-            name: self.name.clone(),
-            value: self.value.clone(),
-        }
-    }
-
     /// Convert a string to a header
     ///
     /// String must be in the format `name: value`
@@ -108,16 +90,6 @@ pub fn headers_to_string(headers: Vec<Header>) -> String {
     headers_string.join("\r\n")
 }
 
-// Impl clone for Header
-impl Clone for Header {
-    fn clone(&self) -> Header {
-        Header {
-            name: self.name.clone(),
-            value: self.value.clone(),
-        }
-    }
-}
-
 impl fmt::Debug for Header {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_struct("Header")
