@@ -9,7 +9,7 @@ Just add the following to your `Cargo.toml`:
 
 ```toml
 [dependencies]
-afire = "0.4.0"
+afire = "1.0.0"
 ```
 
 ## 📄 Info
@@ -59,7 +59,7 @@ For these you will need to enable the features.
 To use these extra features enable them like this:
 
 ```toml
-afire = { version = "0.4.0", features = ["rate_limit", "logging", "serve_static"] }
+afire = { version = "1.0.0", features = ["rate_limit", "logging", "serve_static"] }
 ```
 
 - Content Types
@@ -92,13 +92,15 @@ server.start().unwrap();
 #![warn(missing_docs)]
 
 #[doc(hidden)]
-pub const VERSION: &str = "0.4.0*";
+pub const VERSION: &str = "1.0.0";
 
 // Export Internal Functions
+#[macro_use]
 pub mod internal;
 
 // Import Internal Functions
 mod handle;
+mod thread_pool;
 use internal::common;
 use internal::http;
 use internal::path;
